@@ -37,20 +37,6 @@
 static GObjectClass *parent_class = NULL;
 static GRegex *arv_gv_device_url_regex = NULL;
 
-struct _ArvGvDevicePrivate {
-	gboolean is_controller;
-
-	void *io_data;
-
-	void *heartbeat_thread;
-	void *heartbeat_data;
-
-	ArvGc *genicam;
-
-	char *genicam_xml;
-	size_t genicam_xml_size;
-};
-
 /* Shared data (main thread - heartbeat) */
 
 typedef struct {
@@ -68,8 +54,22 @@ typedef struct {
 
 	unsigned int gvcp_n_retries;
 	unsigned int gvcp_timeout_ms;
+
+	gboolean is_controller;
 } ArvGvDeviceIOData;
 
+struct _ArvGvDevicePrivate {
+	ArvGvDeviceIOData *io_data;
+
+	void *heartbeat_thread;
+	void *heartbeat_data;
+
+	ArvGc *genicam;
+
+	char *genicam_xml;
+	size_t genicam_xml_size;
+};
+
 static gboolean
 _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *buffer)
 {
@@ -79,6 +79,7 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	gboolean error = FALSE;
 
 	answer_size = arv_gvcp_packet_get_read_memory_ack_size (size);
 
@@ -91,11 +92,11 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 						       / sizeof (guint32)) * sizeof (guint32),
 						      0, &packet_size);
 
-	arv_gvcp_packet_debug (packet);
-
 	do {
 		arv_gvcp_packet_set_packet_count (packet, ++io_data->packet_count);
 
+		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
+
 		g_socket_send_to (io_data->socket, io_data->device_address,
 				  (const char *) packet, packet_size,
 				  NULL, NULL);
@@ -104,19 +105,30 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 			count = g_socket_receive (io_data->socket, io_data->buffer,
 						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
 			if (count >= answer_size) {
-				arv_gvcp_packet_debug ((ArvGvcpPacket *) io_data->buffer);
-				if (arv_gvcp_packet_get_packet_count (io_data->buffer) == io_data->packet_count) {
-					memcpy (buffer,
-						arv_gvcp_packet_get_read_memory_ack_data (io_data->buffer),
-						size);
+				ArvGvcpPacket *ack_packet = io_data->buffer;
+				ArvGvcpPacketType packet_type;
+				ArvGvcpCommand command;
+				guint16 packet_count;
+
+				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+				command = arv_gvcp_packet_get_command (ack_packet);
+				packet_count = arv_gvcp_packet_get_packet_count (ack_packet);
+
+				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+				    command == ARV_GVCP_COMMAND_READ_MEMORY_ACK &&
+				    packet_count == io_data->packet_count) {
+					memcpy (buffer, arv_gvcp_packet_get_read_memory_ack_data (ack_packet), size);
 					success = TRUE;
-				}
+				} else
+					error = TRUE;
 			}
 		}
 
 		n_retries++;
 
-	} while (!success && n_retries < io_data->gvcp_n_retries);
+	} while (!success && !error && n_retries < io_data->gvcp_n_retries);
 
 	arv_gvcp_packet_free (packet);
 
@@ -133,6 +145,7 @@ _write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	gboolean error = FALSE;
 
 	g_mutex_lock (io_data->mutex);
 
@@ -143,11 +156,11 @@ _write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *
 
 	memcpy (arv_gvcp_packet_get_write_memory_cmd_data (packet), buffer, size);
 
-	arv_gvcp_packet_debug (packet);
-
 	do {
 		arv_gvcp_packet_set_packet_count (packet, ++io_data->packet_count);
 
+		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
+
 		g_socket_send_to (io_data->socket, io_data->device_address,
 				  (const char *) packet, packet_size,
 				  NULL, NULL);
@@ -156,15 +169,29 @@ _write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *
 			count = g_socket_receive (io_data->socket, io_data->buffer,
 						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
 			if (count >= arv_gvcp_packet_get_write_memory_ack_size ()) {
-				arv_gvcp_packet_debug ((ArvGvcpPacket *) io_data->buffer);
-				if (arv_gvcp_packet_get_packet_count (io_data->buffer) == io_data->packet_count)
+				ArvGvcpPacket *ack_packet = io_data->buffer;
+				ArvGvcpPacketType packet_type;
+				ArvGvcpCommand command;
+				guint16 packet_count;
+
+				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+				command = arv_gvcp_packet_get_command (ack_packet);
+				packet_count = arv_gvcp_packet_get_packet_count (ack_packet);
+
+				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+				    command == ARV_GVCP_COMMAND_WRITE_MEMORY_ACK &&
+				    packet_count == io_data->packet_count)
 					success = TRUE;
+				else
+					error = TRUE;
 			}
 		}
 
 		n_retries++;
 
-	} while (!success && n_retries < io_data->gvcp_n_retries);
+	} while (!success && !error && n_retries < io_data->gvcp_n_retries);
 
 	arv_gvcp_packet_free (packet);
 
@@ -181,16 +208,17 @@ _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_plac
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	gboolean error = FALSE;
 
 	g_mutex_lock (io_data->mutex);
 
 	packet = arv_gvcp_packet_new_read_register_cmd (address, 0, &packet_size);
 
-	arv_gvcp_packet_debug (packet);
-
 	do {
 		arv_gvcp_packet_set_packet_count (packet, ++io_data->packet_count);
 
+		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
+
 		g_socket_send_to (io_data->socket, io_data->device_address,
 				  (const char *) packet, packet_size,
 				  NULL, NULL);
@@ -199,15 +227,30 @@ _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_plac
 			count = g_socket_receive (io_data->socket, io_data->buffer,
 						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
 			if (count > 0) {
-				arv_gvcp_packet_debug ((ArvGvcpPacket *) io_data->buffer);
-				*value_placeholder = arv_gvcp_packet_get_read_register_ack_value (io_data->buffer);
-				success = TRUE;
+				ArvGvcpPacket *ack_packet = io_data->buffer;
+				ArvGvcpPacketType packet_type;
+				ArvGvcpCommand command;
+				guint16 packet_count;
+
+				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+				command = arv_gvcp_packet_get_command (ack_packet);
+				packet_count = arv_gvcp_packet_get_packet_count (ack_packet);
+
+				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+				    command == ARV_GVCP_COMMAND_READ_REGISTER_ACK &&
+				    packet_count == io_data->packet_count) {
+					*value_placeholder = arv_gvcp_packet_get_read_register_ack_value (ack_packet);
+					success = TRUE;
+				} else
+					error = TRUE;
 			}
 		}
 
 		n_retries++;
 
-	} while (!success && n_retries < io_data->gvcp_n_retries);
+	} while (!success && !error && n_retries < io_data->gvcp_n_retries);
 
 	arv_gvcp_packet_free (packet);
 
@@ -227,16 +270,17 @@ _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value)
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	gboolean error = FALSE;
 
 	g_mutex_lock (io_data->mutex);
 
 	packet = arv_gvcp_packet_new_write_register_cmd (address, value, io_data->packet_count, &packet_size);
 
-	arv_gvcp_packet_debug (packet);
-
 	do {
 		arv_gvcp_packet_set_packet_count (packet, ++io_data->packet_count);
 
+		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
+
 		g_socket_send_to (io_data->socket, io_data->device_address, (const char *) packet, packet_size,
 				  NULL, NULL);
 
@@ -244,14 +288,31 @@ _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value)
 			count = g_socket_receive (io_data->socket, io_data->buffer,
 						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
 			if (count > 0) {
-				arv_gvcp_packet_debug ((ArvGvcpPacket *) io_data->buffer);
-				success = TRUE;
+				ArvGvcpPacket *ack_packet = io_data->buffer;
+				ArvGvcpPacketType packet_type;
+				ArvGvcpCommand command;
+				guint16 packet_count;
+
+				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+				command = arv_gvcp_packet_get_command (ack_packet);
+				packet_count = arv_gvcp_packet_get_packet_count (ack_packet);
+
+				arv_log_gvcp ("%d, %d, %d", packet_type, command, packet_count);
+
+				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+				    command == ARV_GVCP_COMMAND_WRITE_REGISTER_ACK &&
+				    packet_count == io_data->packet_count)
+					success = TRUE;
+				else
+					error = TRUE;
 			}
 		}
 
 		n_retries++;
 
-	} while (!success && n_retries < io_data->gvcp_n_retries);
+	} while (!success && !error && n_retries < io_data->gvcp_n_retries);
 
 	arv_gvcp_packet_free (packet);
 
@@ -277,8 +338,22 @@ arv_gv_device_heartbeat_thread (void *data)
 
 	do {
 		g_usleep (thread_data->period_us);
-		_read_register (io_data, ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE, &value);
-		arv_debug ("device", "[GvDevice::Heartbeat] (%d)", value);
+
+		if (io_data->is_controller) {
+
+			/* TODO: Instead of reading the control register, Pylon does write the heartbeat
+			 * timeout value, which is interresting, as doing this we could get an error
+			 * ack packet which will indicate we lost the control access. */
+
+			_read_register (io_data, ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_OFFSET, &value);
+			arv_log_device ("[GvDevice::Heartbeat] (%d)", value);
+
+			if ((value & (ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_CONTROL |
+				      ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_EXCLUSIVE)) == 0) {
+				arv_warning_device ("[GvDevice::Heartbeat] Control access lost");
+				io_data->is_controller = FALSE;
+			}
+		}
 	} while (!thread_data->cancel);
 
 	return NULL;
@@ -289,22 +364,30 @@ arv_gv_device_heartbeat_thread (void *data)
 static gboolean
 arv_gv_device_take_control (ArvGvDevice *gv_device)
 {
-	gv_device->priv->is_controller = arv_device_write_register (ARV_DEVICE (gv_device),
-								    ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE, 2);
+	gboolean success;
 
-	return gv_device->priv->is_controller;
+	success = arv_device_write_register (ARV_DEVICE (gv_device),
+					     ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_OFFSET,
+					     ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_CONTROL);
+
+	gv_device->priv->io_data->is_controller = success;
+
+	if (!success)
+		arv_warning_device ("[GvDevice::take_control] Can't get control access");
+
+	return success;
 }
 
 static gboolean
 arv_gv_device_leave_control (ArvGvDevice *gv_device)
 {
-	gboolean result;
+	gboolean success;
 
-	result = arv_device_write_register (ARV_DEVICE (gv_device),
-					    ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE, 0);
-	gv_device->priv->is_controller = FALSE;
+	success = arv_device_write_register (ARV_DEVICE (gv_device),
+					    ARV_GVBS_CONTROL_CHANNEL_PRIVILEGE_OFFSET, 0);
+	gv_device->priv->io_data->is_controller = FALSE;
 
-	return result;
+	return success;
 }
 
 guint64
@@ -316,10 +399,10 @@ arv_gv_device_get_timestamp_tick_frequency (ArvGvDevice *gv_device)
 	g_return_val_if_fail (ARV_IS_GV_DEVICE (gv_device), 0);
 
 	if (arv_device_read_register (ARV_DEVICE (gv_device),
-				      ARV_GVBS_TIMESTAMP_TICK_FREQUENCY_HIGH,
+				      ARV_GVBS_TIMESTAMP_TICK_FREQUENCY_HIGH_OFFSET,
 				      &timestamp_tick_frequency_high) &&
 	    arv_device_read_register (ARV_DEVICE (gv_device),
-				      ARV_GVBS_TIMESTAMP_TICK_FREQUENCY_LOW,
+				      ARV_GVBS_TIMESTAMP_TICK_FREQUENCY_LOW_OFFSET,
 				      &timestamp_tick_frequency_low)) {
 		guint64 timestamp_tick_frequency;
 
@@ -339,12 +422,23 @@ arv_gv_device_get_packet_size (ArvGvDevice *gv_device)
 	g_return_val_if_fail (ARV_IS_GV_DEVICE (gv_device), 0);
 
 	arv_device_read_register (ARV_DEVICE (gv_device),
-				  ARV_GVBS_FIRST_STREAM_CHANNEL_PACKET_SIZE,
+				  ARV_GVBS_STREAM_CHANNEL_0_PACKET_SIZE_OFFSET,
 				  &packet_size);
 
 	return packet_size;
 }
 
+void
+arv_gv_device_set_packet_size (ArvGvDevice *gv_device, guint packet_size)
+{
+	g_return_if_fail (ARV_IS_GV_DEVICE (gv_device));
+	g_return_if_fail (packet_size > 0);
+
+	arv_device_write_register (ARV_DEVICE (gv_device),
+				  ARV_GVBS_STREAM_CHANNEL_0_PACKET_SIZE_OFFSET,
+				  packet_size);
+}
+
 static char *
 _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 {
@@ -361,7 +455,7 @@ _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 
 	filename[ARV_GVBS_XML_URL_SIZE - 1] = '\0';
 
-	arv_debug ("device", "[GvDevice::load_genicam] xml url = '%s' at 0x%x", filename, address);
+	arv_debug_device ("[GvDevice::load_genicam] xml url = '%s' at 0x%x", filename, address);
 
 	tokens = g_regex_split (arv_gv_device_url_regex, filename, 0);
 
@@ -378,9 +472,8 @@ _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 			file_address = strtoul (tokens[3], NULL, 16);
 			file_size = strtoul (tokens[4], NULL, 16);
 
-			arv_debug ("device",
-				   "[GvDevice::load_genicam] Xml address = 0x%x - size = 0x%x - %s",
-				   file_address, file_size, tokens[2]);
+			arv_debug_device ("[GvDevice::load_genicam] Xml address = 0x%x - size = 0x%x - %s",
+					  file_address, file_size, tokens[2]);
 
 			if (file_size > 0) {
 				genicam = g_malloc (file_size);
@@ -392,8 +485,7 @@ _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 						ArvZip *zip;
 						const GSList *zip_files;
 
-						arv_debug ("device",
-							   "[GvDevice::load_genicam] Zipped xml data");
+						arv_debug_device ("[GvDevice::load_genicam] Zipped xml data");
 
 						zip = arv_zip_new (genicam, file_size);
 						zip_files = arv_zip_get_file_list (zip);
@@ -411,8 +503,7 @@ _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 							file_size = tmp_buffer_size;
 							genicam = tmp_buffer;
 						} else
-							arv_debug ("device",
-								   "[GvDevice::load_genicam] Invalid format");
+							arv_warning_device ("[GvDevice::load_genicam] Invalid format");
 						arv_zip_free (zip);
 					}
 					*size = file_size;
@@ -444,9 +535,9 @@ arv_gv_device_get_genicam_xml (ArvDevice *device, size_t *size)
 
 	*size = 0;
 
-	xml = _load_genicam (gv_device, ARV_GVBS_FIRST_XML_URL, size);
+	xml = _load_genicam (gv_device, ARV_GVBS_XML_URL_0_OFFSET, size);
 	if (xml == NULL)
-		xml = _load_genicam (gv_device, ARV_GVBS_SECOND_XML_URL, size);
+		xml = _load_genicam (gv_device, ARV_GVBS_XML_URL_1_OFFSET, size);
 
 	gv_device->priv->genicam_xml = xml;
 	gv_device->priv->genicam_xml_size = *size;
@@ -476,29 +567,40 @@ arv_gv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void
 	const guint8 *address_bytes;
 	guint32 stream_port;
 	guint packet_size;
+	guint32 n_stream_channels;
 	GInetAddress *interface_address;
 	GInetAddress *device_address;
 
+	arv_device_read_register (device, ARV_GVBS_N_STREAM_CHANNELS_OFFSET, &n_stream_channels);
+	arv_debug_device ("[GvDevice::create_stream] Number of stream channels = %d", n_stream_channels);
+
+	if (n_stream_channels < 1)
+		return NULL;
+
+	if (!io_data->is_controller) {
+		arv_warning_device ("[GvDevice::create_stream] Can't create stream without control access");
+		return NULL;
+	}
+
 	interface_address = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (io_data->interface_address));
 	device_address = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (io_data->device_address));
 	address_bytes = g_inet_address_to_bytes (interface_address);
 
+	arv_gv_device_set_packet_size (gv_device, ARV_GV_DEVICE_GVSP_PACKET_SIZE_DEFAULT);
 	packet_size = arv_gv_device_get_packet_size (gv_device);
 
-	arv_debug ("device", "[GvDevice::create_stream] Packet size = %d byte(s)", packet_size);
+	arv_debug_device ("[GvDevice::create_stream] Packet size = %d byte(s)", packet_size);
 
 	stream = arv_gv_stream_new (device_address, 0, callback, user_data,
 				    arv_gv_device_get_timestamp_tick_frequency (gv_device), packet_size);
 
 	stream_port = arv_gv_stream_get_port (ARV_GV_STREAM (stream));
 
-	arv_device_write_register (device, ARV_GVBS_FIRST_STREAM_CHANNEL_PACKET_SIZE, 0x000005dc);
-	arv_device_write_register (device, ARV_GVBS_FIRST_STREAM_CHANNEL_IP_ADDRESS,
+	arv_device_write_register (device, ARV_GVBS_STREAM_CHANNEL_0_IP_ADDRESS_OFFSET,
 				   g_htonl(*((guint32 *) address_bytes)));
-	arv_device_write_register (device, ARV_GVBS_FIRST_STREAM_CHANNEL_PORT, stream_port);
-	arv_device_read_register (device, ARV_GVBS_FIRST_STREAM_CHANNEL_PORT, &stream_port);
+	arv_device_write_register (device, ARV_GVBS_STREAM_CHANNEL_0_PORT_OFFSET, stream_port);
 
-	arv_debug ("device", "[GvDevice::create_stream] stream port = %d", stream_port);
+	arv_debug_device ("[GvDevice::create_stream] Stream port = %d", stream_port);
 
 	return stream;
 }
@@ -575,15 +677,15 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 	g_return_val_if_fail (G_IS_INET_ADDRESS (device_address), NULL);
 
 	address_string = g_inet_address_to_string (interface_address);
-	arv_debug ("device", "[GvDevice::new] Interface address = %s", address_string);
+	arv_debug_device ("[GvDevice::new] Interface address = %s", address_string);
 	g_free (address_string);
 	address_string = g_inet_address_to_string (device_address);
-	arv_debug ("device", "[GvDevice::new] Device address = %s", address_string);
+	arv_debug_device ("[GvDevice::new] Device address = %s", address_string);
 	g_free (address_string);
 
 	gv_device = g_object_new (ARV_TYPE_GV_DEVICE, NULL);
 
-	io_data = g_new (ArvGvDeviceIOData, 1);
+	io_data = g_new0 (ArvGvDeviceIOData, 1);
 
 	io_data->mutex = g_mutex_new ();
 	io_data->packet_count = 0;
@@ -602,7 +704,6 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 	io_data->poll_in_event.events =  G_IO_IN;
 	io_data->poll_in_event.revents = 0;
 
-
 	gv_device->priv->io_data = io_data;
 
 	arv_gv_device_load_genicam (gv_device);
